package com.rocket9labs.boxcars;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.view.KeyEvent;
import android.view.inputmethod.BaseInputConnection;
import android.view.inputmethod.InputMethodManager;

import com.rocket9labs.boxcars.mobile.EbitenView;

import go.Seq;

public class MainActivity extends Activity {
    private static final int backButtonLongPress = 2000; // Milliseconds.
    private long backButtonPressed;
    private long keyPressed;
    private final Handler handler = new Handler();
    private final Runnable exitRunnable = new Runnable() {
        @Override
        public void run() {
            System.exit(0);
        }
    };

    private EbitenView getEbitenView() {
        return this.findViewById(R.id.ebitenView);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Seq.setContext(getApplicationContext());
        setContentView(R.layout.activity_main);
        getEbitenView().requestFocus();
    }

    @Override
    protected void onPause() {
        handler.postDelayed(exitRunnable, 10 * 60 * 1000);
        getEbitenView().suspendGame();
        super.onPause();
    }

    @Override
    protected void onResume() {
        handler.removeCallbacks(exitRunnable);
        super.onResume();
        getEbitenView().resumeGame();
        getEbitenView().requestFocus();
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            backButtonPressed = System.currentTimeMillis();
            event.startTracking();
            return true;
        }
        boolean handled = super.onKeyDown(keyCode, event);
        getEbitenView().requestFocus();
        return handled;
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            if (System.currentTimeMillis() - backButtonPressed >= backButtonLongPress) {
                finishAndRemoveTask();
                System.exit(0);
                return true;
            }
            InputMethodManager imm = (InputMethodManager) getSystemService(Activity.INPUT_METHOD_SERVICE);
            imm.toggleSoftInputFromWindow(getEbitenView().getWindowToken(), 0, 0);
            getEbitenView().requestFocus();
            return true;
        }
        boolean handled = super.onKeyDown(keyCode, event);
        getEbitenView().requestFocus();
        return handled;
    }

    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        if (event.getAction() == KeyEvent.ACTION_UP) {
            int keycode = event.getKeyCode();
            if (keycode == 66 || keycode == 67 || (keycode >= 7 && keycode <= 16 && !event.isShiftPressed())) {
                long now = System.currentTimeMillis();
                if (now - keyPressed >= 100) {
                    super.dispatchKeyEvent(event);
                    BaseInputConnection mInputConnection = new BaseInputConnection((EbitenView) findViewById(R.id.ebitenView), false);
                    if (keycode == 66) {
                        mInputConnection.commitText(Character.toString((char) 199), 1);
                    } else if (keycode == 67) {
                        mInputConnection.commitText(Character.toString((char) 231), 1);
                    } else {
                        mInputConnection.commitText(Character.toString((char) (48 + (keycode - 7))), 1);
                    }
                    keyPressed = now;
                    getEbitenView().requestFocus();
                    return true;
                }
            }
        }
        boolean handled = super.dispatchKeyEvent(event);
        getEbitenView().requestFocus();
        return handled;
    }
}